define([
		"dojo/_base/declare",
		"dojo/_base/lang",
		"dojox/mvc/getPlainValue",
		"dojox/mvc/at",
		"dojo/promise/all",
		"dojox/mvc/getStateful",
		"aps/ResourceStore",
		"aps/Memory",
		"aps/TextBox",
		"aps/Tile",
		"aps/FieldSet",
		"aps/Output",
		"aps/json!./wizard/newvps.json",
		"aps/json!./newoffer.json",
		"aps/View"
	],
	function (declare, lang, getPlainValue, at, all, getStateful,
			Store, Memory, TextBox, Tile, FieldSet, Output,
			newVPS, newOffer, View) {
		var self;
		return declare(View, {
			init: function() {
				self = this;
				/* Declare the data sources */
				self.userList = new Memory({data: [], idProperty: "id"});   // List of new users

				/* Render a gird cell to allow entering a host name */
				var renderHostName = function(row) {
				   return new TextBox({
				      value: at(row, "hostName"),
				      required: true
				   });
				};

				/* Define and return widgets */
				return [
					["aps/Grid",{
						id: self.genId("addUsers_grid"),
						store:   self.userList,
						columns: [
							{ name: "User Name", field: "name" },
							{ name: "Host Name", field: "hostName", renderCell: renderHostName }
						]
					}],
					["aps/Tiles", {
						id: self.genId("addUsers_offerSet"),
						title: "Select an offer",
						selectionMode: "single",
						required: true
					}]
				];
            }, // End of Init

            /* Process data available at the onContext phase */
            onContext: function() {
				/* Define data for the APS Biz API */
				aps.context.subscriptionId = aps.context.vars.context.aps.subscription;

				var offerStore = new Store({
					target: "/aps/2/resources",
					apsType: "http://aps-standard.org/samples/srv-activation/offer/1.0"
				});

				var offerFilter = { filter:
					[{ apsType: "http://aps-standard.org/samples/srv-activation/offer/1.0" }]
				};

				/* Retrieve the resources to be processed */
				all([
				     aps.biz.getResourcesToBind(), // New users
				     aps.biz.getResourcesInfo(offerFilter), // Subscribed offers
				     offerStore.query()	// All offers of the application
				]).then(function(data) {
					if(data[1].length === 0) return; // No offers in the subscription
					
					/* User processing */
					self.userList.setData(data[0].users.map(function(user) {
						return getStateful({
							id: user.aps.id,
							name: user.fullName,
							hostName: ""
						});
					}));
					self.byId("addUsers_grid").set("store", self.userList);

					/* Prepare the Offer-by-ID dictionary */
					self.offerById = {};
					data[2].forEach(function(offer) {
						self.offerById[offer.aps.id] = offer;
					});

					/* Define the offer selection list */
					var offerCache = new Memory({data: [], idProperty: "aps.id"});
					offerCache.setData(data[1].map(function(offer) {
						var available = (offer.limit - offer.usage >= self.userList.data.length) ? true : false;
						var offerApsResource = self.offerById[offer.apsId];
						return getStateful({
							id: offer.apsId,
							name: offerApsResource.name,
							os: offerApsResource.platform.OS.name,
							cpu: offerApsResource.hardware.CPU.number,
							memory: offerApsResource.hardware.memory,
							diskspace: offerApsResource.hardware.diskspace,
							limit: offer.limit,
							usage: offer.usage,
							priceText: offer.priceText,
							available: available
						});
					}));

					/* Widget definition */
                    var parentContainer = self.byId("addUsers_offerSet"); // ``aps/Tiles``
                    parentContainer.removeAll();	// Remove all child tiles
                    offerCache.data.forEach(function(offer) {
						var availability = (offer.available) ? "Available for the selected number of users." :
							"Not available for the selected number of users.";

						var tile = new Tile({
							offerId: offer.id,
							gridSize: "md-4 xs-12",
							title: offer.name,
							state: offer.available ? "" : "inactive",
							disabled: offer.available ? false : true
						});
						var tileBody = new FieldSet();
						var outputSet = [
							new Output({
								value: offer.os,
								content: "Operating System: ${value}"
							}),
							new Output({
								value: offer.cpu,
								content: "CPU cores: ${value}"
							}),
							new Output({
								value: offer.memory,
								content: "Memory: ${value} MB"
							}),
							new Output({
								value: offer.diskspace,
								content: "Disk space: ${value} GB"
							}),
							new Output({
								value: offer.priceText
							}),
							new Output({
								value: availability
							}),
						    new Output({
								usage: offer.usage,
								limit: offer.limit,
								content: "${usage} of ${limit} servers are used."
						    })
						];
						outputSet.forEach(function(widget) {
							tileBody.addChild(widget);
						});
						tile.addChild(tileBody);
						parentContainer.addChild(tile);
                    });
				})	// End of all-then callback function

				/* Hide loading once all async operations are completed */
				.then(aps.apsc.hideLoading());
            }, // End of onContext

            /* Create the handler for the Next navigation button */
            onNext: function() {
				/* Clear out all messages and validate the data */
				var page = this.byId("apsPageContainer");
				page.get("messageList").removeAll();
				if (!page.validate()) {
					aps.apsc.cancelProcessing();
					return;
				}

				/* Collect data for VPS provisioning */
				var selectedOfferTileId = self.byId("addUsers_offerSet").get("selectionArray")[0];
				var selectedOfferTile = self.byId(selectedOfferTileId);
				var offerId = selectedOfferTile.get("offerId");
				var offer = self.offerById[offerId];
				var users = self.userList.data;

				/* Common VPS model based on the selected offer */
				var vpsModel = getStateful({ "data": newVPS }); // Default VPS properties
                vpsModel.data.set({
					offer: {aps: {id: offerId}},
					platform: offer.platform,
					hardware: offer.hardware
                });

				/* Call the APS Biz API to provision VPSes for the new users*/
                aps.biz.requestUsageChange({
					deltas: [{
						apsType: "http://aps-standard.org/samples/srv-activation/vps/1.0",
						delta: users.length
					}],
					operations: users.map(function(user) {
						return {
							provision: lang.mixin(getPlainValue(vpsModel.data), {
								name: user.hostName,
								user: {aps: {id: user.id}},
								userName: user.name
							})
						};
					})
				}).then(function() {
					aps.apsc.next();
				});
            }

	});  // End of Declare

});      // End of Define
