define([ "dojo/_base/declare",
			"aps/ResourceStore",
			"dojo/when",
			"dijit/registry",
			"aps/load",
			"aps/xhr",
			"./displayError.js",
			"dojo/_base/array",
         "aps/_View" ],
function(declare, Store, when, registry, load, xhr, displayError, arr, _View) {
    var page, grid;
	return declare(_View, {
        init: function() {
			/* Create the data store */
			var store = new Store({
				apsType: "http://aps-standard.org/samples/srv-activation/vps/1.0",
				target: "/aps/2/resources/"
			});

			/* Widget processing logic */
			var refresh = function() {
				grid.refresh();
			};

			function changeState(state, btn) {
				var sel = grid.get("selectionArray"),
				counter = sel.length;

				/* Clear the current messages on the screen */
				page.get("messageList").removeAll();

				arr.forEach(sel, function(vpsId){
					console.log("I'm trying to change state of VPS with id = [" + vpsId + "]");

					/* Save the VPS properties */
					when(xhr("/aps/2/resources/" + vpsId + "/" + state, {method: "GET",  handleAs: "text"}),
						 /* If success, process the next VPS until the list is empty */
						function(){
							console.log("State of VPS with id = [" + vpsId + "] changed");
							sel.splice(sel.indexOf(vpsId),1);
							grid.refresh();
							if (--counter === 0) { btn.cancel(); } /* Remove busy state for button */
						},
						/* If failure, call the error handler */
						function(e){
							displayError(e);
                            if (--counter === 0) { btn.cancel(); }
						}
					);
				});
			}

			/* Create the on-click handler for the *Stop* button */
			var stop = function() {
				changeState("stop", this);
			};

			/* Create the on-click handler for the *Start* button */
			var start = function() {
				changeState("start", this);
			};

			/* Define and return widgets */
			return ["aps/Grid", {
				id:                "mysrv_grid",
				store:             store,
			    selectionMode:     "multiple",
				columns:           [
					{ field: "name",				name: "Name"},
					{ field: "hardware.memory",		name: "RAM",       title: "RAM" },
					{ field: "hardware.diskspace",	name: "Disk Space", title: "Disk" },
					{ field: "hardware.CPU.number",	name: "CPU" },
					{ field: "platform.OS.name",	name: "OS" },
					{ field: "state",				name: "State" },
					{ field: "aps.status",			name: "APS Status" }

				]
		    }, [
                ["aps/Toolbar", [
					["aps/ToolbarButton", {
						id: "mysrv_start",
						iconClass:"fa-play",
						type: "success",
						label: "Start",
						requireItems: true,
						onClick: start
					}],
					["aps/ToolbarButton", {
						id: "mysrv_stop",
						iconClass:"fa-pause",
						type: "warning",
						label: "Stop",
						requireItems: true,
						onClick: stop
					}],
					["aps/ToolbarButton", {
						id: "mysrv_refresh",
						iconClass:"fa-refresh",
						label: "Refresh",
						autoBusy: false,
						onClick: refresh
					}]
				]]]];
        }, // End of Init
			
        onShow: function() {
            registry.byId("mysrv_grid").refresh();
        },
        onContext: function() {
			grid = grid || registry.byId("mysrv_grid");
			page = page || registry.byId("apsPageContainer");
			aps.apsc.hideLoading();
        }
   });		// End of Declare
});			// End of Define
