define([
    "aps/nav/ViewPlugin",
    "dojo/_base/declare",
    "dojo/Deferred",
    "dojox/mvc/getStateful",
    "aps/FieldSet",
    "aps/Container",
    "aps/Output",
    "aps/Gauge"
], function(ViewPlugin, declare, Deferred, getStateful, FieldSet, Container, Output, Gauge) {

    return declare(ViewPlugin, {
        init: function(mediator) {
			mediator.getServicesList = function() {
				return { "title": "VPSes" };
			};

            mediator.getState = function(users) {
				/* Process each user and add the result to userServiceMap */
				var userStates = users.reduce(function(userServiceMap, user) {
					var state = {};
					state.status = "Not Assigned";
					state.title = "VPSes";
					var vpses = 0,
						stopped = 0,
						running = 0;
	                /* Process each service assigned to the user */
	                user.services.forEach(function(service) {
						/* Account only the services provided by this App - VPSes */
						if (service.aps.type === "http://aps-standard.org/samples/srv-activation/vps/1.0") {
	                        state.status = "Assigned";
	                        vpses += 1;
	                        if (service.state === "Stopped") {
	                            stopped += 1;
	                        }
	                        else running += 1;
	                    }
	                });
	                if(state.status === "Assigned") {
						/* Create an aps/Container widget for the Tile mode in the Users list*/
						if (!state.widget) {
							var vpsOutput = new Container();
							vpsOutput.addChild(new Output({
								value: vpses,
								running: running,
								stopped: stopped,
								content: "Assigned: ${value} <br/> Running: ${running} <br/> Stopped: ${stopped}"
							}));
	                        state.widget = vpsOutput;
	                    }
						/* Create an aps/Gauge widget for the Grid mode in the Users list*/
						if (!state.gridWidget) {
	                        var vpsGridOutput = new Gauge({
	                            legend : "${value} of ${maximum} are stopped",
	                            minimum : 0,
	                            maximum : vpses,
	                            value: stopped
	                        });
	                        state.gridWidget = vpsGridOutput;
	                    }
	                }
	                userServiceMap[user.aps.id] = [state];  // Generally, it can be [state1, ...]
	                return userServiceMap;
                }, {});	// End of Reduce, initial value of userServiceMap is {}
				return userStates;
            };	// End of getState
        }	// End of Init 
    });	// End of Declare
});	// End of Define
