define([
  "dojo/_base/declare",
  "dojo/when",
  "dojo/promise/all",
  "dojox/mvc/getPlainValue",
  "dojox/mvc/at",
  "dojox/mvc/getStateful",
  
  "aps/View",
  "aps/ResourceStore",
  "aps/Memory",
  "aps/xhr",
  
  "aps/json!./wizard/newvps.json",
  "aps/json!./newoffer.json",
  "./displayError"
], function (declare, when, all, getPlainValue, at, getStateful, View, Store,
	Memory, xhr, newVPS, newOffer, displayError) {
	var self;
	return declare(View, {
		init: function () {
			self = this;
			self.offerCache = new Memory({
			   "data": [],
			   idProperty: "aps.id"
			});
			self.offerCtrl = getStateful({
			   "data": newOffer
			});
			
			self.vpsModel = getStateful({"data": newVPS});
			
			function onChangeOffer(offerID) {
			   console.log("Changed for offer: " + offerID); // For debugging only
			   when(self.offerCache.get(offerID), function(data) {
			      self.offerCtrl.data.set(data);  // Update the current offer
			   });
			}
			
			/* Define and return widgets */
			return ["aps/Panel", {id: this.genId("edit_form")}, [

				["aps/FieldSet", {
						title: "General"
					},
					[
						["aps/TextBox", {
							label: "Server Name",
							value: at(self.vpsModel.data, "name"),
							required: true
						}],
						["aps/TextBox", {
							label: "Description",
							value: at(self.vpsModel.data, "description")
						}]
					]
				],
				["aps/Panel", {id: this.genId("srvEdit_offers_panel"), title: "Offer"}, [
					["aps/FieldSet", {id: this.genId("srvEdit_offers")}, [
						["aps/Select", {
							id: this.genId("srvEdit_offerSelect"),
							gridSize: 'md-3 xs-12',
							store: self.offerCache,
							required: true,
							label: "Bundle",
							onChange: onChangeOffer,
							labelAttr: "name",
							value: at(self.vpsModel.data.offer.aps, "id")
						}]
					]]
				]],
				["aps/FieldSet", {
						title: "Server Configuration"
					}, [
						["aps/Output", {
							   id: this.genId("srvEdit_os"),
							   label: "OS",
							   value: at(self.offerCtrl.data.platform.OS, "name")
						}],
						["aps/Spinner", {
							label: "CPU Number",
							minimum: 1,
							maximum: at(self.offerCtrl.data.hardware.CPU, "number"),
							value: at(self.vpsModel.data.hardware.CPU, "number"),
							step: 1
						}],
						["aps/Spinner", {
							label: "Disk Space",
							minimum: 1,
							maximum: at(self.offerCtrl.data.hardware, "diskspace"),
							value: at(self.vpsModel.data.hardware, "diskspace"),
							legend: "Gb",
							step: 1
						}],
						["aps/Spinner", {
							label: "RAM",
							minimum: 128,
							maximum: at(self.offerCtrl.data.hardware, "memory"),
							value: at(self.vpsModel.data.hardware, "memory"),
							legend: "Mb",
							step: 128
						}]
				]]
			]];
		},
		
		/* Define handlers for the navigation buttons */
		onCancel: function() {
			aps.apsc.gotoView("servers");
		},
		onSubmit: function() {
			aps.context.subscriptionId = aps.context.vars.vps.aps.subscription;
			var vpsStore = new Store({ // Interface with the APS controller
			   apsType: "http://aps-standard.org/samples/srv-activation/vps/1.0",
			   target: "/aps/2/resources/"
			});

			var vps = getPlainValue(self.vpsModel.data), // New properties to save
			   reqs = []; /* We may have one or two REST requests */

			/* If the offer has to be changed, we need to first request re-linking the VPS
			   to the other offer */
			if(vps.offer.aps.id != aps.context.vars.vps.offer.aps.id)
			   reqs.push(xhr("/aps/2/resources/" + vps.aps.id + "/offer",
			      { method: "POST",
			         data: JSON.stringify({ "aps": { "id": vps.offer.aps.id } })
			      }
			   ));
			vps.platform.OS.name = self.offerCtrl.data.platform.OS.name;
			reqs.push(vpsStore.put(vps)); /* And now update the VPS properties */
			all(reqs).then(function() {    /* Once the requests are completed,
			                                 navigate to the "servers" view */
					aps.apsc.gotoView("servers");
			   },
			   displayError
			);

		},
		
		/* Define the view flow methods */
		onContext: function(context) {
	        /* Update the model with actual data from the navigation variable */
			var offerStore = new Store({ // The list of offers for selection
			   apsType: "http://aps-standard.org/samples/srv-activation/offer/1.0",
			   target: "/aps/2/resources/"
			});

			self.vpsModel.data.set(context.vars.vps);
			self.offerCtrl.data.set(context.vars.vps.offer);

			offerStore.query().then(function(offers) {
			   self.offerCache.setData(offers);
			   self.byId("srvEdit_offerSelect").set("store", self.offerCache);
			}).then(function() {
			   aps.apsc.hideLoading();
			});
		}
	});
});
